@@ -51,6 +51,11 @@ class LeverSyntaxError(LeverException):
     code = 400
 
 
+class LeverAccessDenied(LeverException):
+    code = 404
+    pass
+
+
 class LeverNotFound(LeverException):
     code = 404
     pass
@@ -125,11 +130,11 @@ class ImpersonateMixin(object):
     when need arises. Depends on the user_model attribute being set. Allows
     lookup of user that you're acting on the behalf of via __username parameter
     or __user_id parameter. """
-    from flask.ext.login import current_user
     user_model = None
 
     @preprocess(method='post', pri=1000)
     def impersonate(self):
+        from flask.ext.login import current_user
         if self.user_model is None:
             raise Exception("user_model must be defined to lookup users")
         # check to ensure the user can create for others if requested
@@ -255,7 +260,7 @@ class API(six.with_metaclass(APIMeta, MethodView)):
             msg = 'Incorrect syntax or missing key ' + str(e)
             code = 400
             extra = {'key': str(e)}
-        except AttributeError:
+        except (AttributeError, ValueError):
             msg = 'Incorrect syntax or missing key'
             code = 400
         except AssertionError:
@@ -341,7 +346,7 @@ class API(six.with_metaclass(APIMeta, MethodView)):
         return jsonify(**retval)
 
     def post(self):
-        """ Create a new object """
+        """ Perform an action on an object or class """
         self.params = request.get_json(silent=True) or {}
         self.action = self.params.pop('__action', None)
         cls = self.params.pop('__cls', None)
@@ -350,6 +355,8 @@ class API(six.with_metaclass(APIMeta, MethodView)):
             self.action = self.create_method
         for method in self._pre_method.get('post', []):
             method(self)
+        for method in self._pre_method.get(self.action, []):
+            method(self)
 
         if not cls:
             obj = self.get_obj()
@@ -367,11 +374,8 @@ class API(six.with_metaclass(APIMeta, MethodView)):
                 obj = obj(**self.params)
                 self.session.add(obj)
                 self.session.flush()
-                ret = {'objects': [get_joined(obj)]}
             else:
                 ret = getattr(obj, self.action)(**self.params)
-                if isinstance(ret, self.model):
-                    ret = {'objects': [get_joined(ret)]}
         except TypeError as e:
             if 'argument' in str(e):
                 msg = ("Wrong number of arguments supplied for action {0}."
@@ -380,18 +384,24 @@ class API(six.with_metaclass(APIMeta, MethodView)):
             else:
                 raise
 
+        for method in self._post_action.get(self.action, []):
+            ret = method(self, ret)
+        for method in self._post_method.get('post', []):
+            ret = method(self, ret)
+
         if ret is None or ret is True:
             retval['success'] = True
         elif ret is False:
             retval['success'] = False
         else:
+            if hasattr(ret, '__table__'):
+                self.session.flush()
+                ret = {'objects': [get_joined(ret)]}
             retval['success'] = True
             retval.update(ret)
 
         self.session.commit()
 
-        for method in self._post_method.get('post', []):
-            method(self, retval)
         try:
             return jsonify(**retval)
         except (LeverException, KeyError, AttributeError):
@@ -466,7 +476,8 @@ class API(six.with_metaclass(APIMeta, MethodView)):
         elif pg_size is None:  # default to max_pg_size
             pg_size = self.max_pg_size
         pg_size = min(pg_size, self.max_pg_size)  # limit their option to max
-        page = self.params.get('pg', 1)
+        page = int(self.params.get('pg', 1))
+        print (page - 1) * pg_size
         return query.offset((page - 1) * pg_size).limit(pg_size)
 
     def search(self, query=None):
@@ -631,7 +642,12 @@ def jsonize(obj, args, raw=False):
     representation.  For passing back object state via the api """
     dct = {}
     for key in args:
-        attr = getattr(obj, key)
+        try:
+            attr = getattr(obj, key)
+        except Exception:
+            raise LeverServerError(
+                "Invalid join property {0} defined by join profile"
+                .format(key))
         # if it's a callable function call it, then do the parsing below
         if callable(attr):
             try:
